/*	$Id: lalr.h,v 1.1.1.1 1999/10/05 23:58:37 leonard Exp $	*/
/* Copyright 1999, David Leonard. The University of Queensland, Australia. */

/* Production in a grammar. */
typedef struct production {
	void *		p_context;	/* reduce context */
	struct production * p_next_prod;
	struct production * p_next_derive;
	symbol_t	p_lhs;		/* left hand side */
	int		p_nrhs;		/* length of right hand side */
	symbol_t 	p_rhs[MAXRHS];	/* right hand side */
	int		p_prec;		/* precedence level */
} production_t;

struct state; /* fwd decl */

/* Item in a state */
typedef struct item {
	struct item *	i_next;
	production_t *	i_prod;		/* production */
	int		i_dot;		/* position of dot */
	symbol_t	i_follow;	/* follows symbol */
} item_t;
#define IS_REDUCE(i)	((i)->i_dot == (i)->i_prod->p_nrhs)

/* State in a DFA */
typedef struct state {
	struct state *	s_next;
	struct state *	s_next_new;
	item_t *	s_item;		/* items in this state */
	int		s_nkitem;	/* leading items form kernel */
	struct state *	s_shift[MAXSYMS]; /* shift action */
	production_t *	s_reduce[MAXSYMS]; /* reduce action */
	int		s_id;		/* state id number */
} state_t;
#define IS_NEW_STATE(st) ((st)->s_next_new != (st))

typedef enum {
	NONASSOC, LEFTASSOC, RIGHTASSOC
} assoc_t;

/* The grammar */
typedef struct grammar {
	int		g_terminals;
	int		g_nonterminals;
	int		g_nprod;
	production_t *	g_prod;			/* follow p_next_prod */
	production_t *	g_derive[MAXSYMS];	/* follow p_next_derive */
	symset_t 	g_first[MAXSYMS];
	int 		g_symprec[MAXSYMS];	/* precedence of symbols */
	assoc_t		g_symassoc[MAXSYMS];	/* associativity of symbols */
	char **		g_name;			/* symbol names */
	state_t *	g_state;		/* list of states */
	state_t *	g_laststate;		/* last state */
	state_t *	g_newstates;		/* list of new states */
	state_t *	g_init;			/* initial state */
	int		g_rrconflicts;
	int		g_srconflicts;
} grammar_t;
#define IS_NONTERMINAL(g, s)  ((s) >= (g)->g_terminals)
#define IS_TERMINAL(g, s)     ((s) > EPSILON && (s) < (g)->g_terminals)

/* lalr.c */
void	derive_productions(grammar_t *);
int	derive_first(grammar_t *, int, const symbol_t *, symset_t *);
void	derive_firsts(grammar_t *);
void	closure(grammar_t *, state_t *);
void	add_transitions(grammar_t *, state_t *);
state_t *find_state(grammar_t *, item_t *);
state_t *new_state(grammar_t *, item_t *);
void	free_state(state_t *);
item_t *new_item(production_t *, int, symbol_t);
void	free_item(item_t *);
void	free_item_list(item_t *);

/* print.c */
void	print_symset(grammar_t *, symset_t *);
void	print_production(grammar_t *, production_t *);
void	print_item(grammar_t *, item_t *);
void	print_state(grammar_t *, state_t *);

/* misc */
void	nocore(grammar_t *);
