/*	$Id: print.c,v 1.1.1.1 1999/10/05 23:58:37 leonard Exp $	*/
/* Copyright 1999, David Leonard. The University of Queensland, Australia. */

#include <stdio.h>
#include "symbol.h"
#include "lalr.h"

#ifndef NDEBUG

void
print_symset(g, ss) 
	grammar_t *g;
	symset_t *ss;
{
	symbol_t s;
	int comma = 0;

	fprintf(stderr, "[");
	for (s = 0; s < g->g_nonterminals; s++)
	    if (symsetin(ss, s)) {
		fprintf(stderr, "%s%s", comma ? ", ": "", g->g_name[s]);
		comma = 1;
	    }
	fprintf(stderr, "]");
}

void
print_production(g, p)
	grammar_t *g;
	production_t *p;
{
	int i;

	fprintf(stderr, "%s ->", g->g_name[p->p_lhs]);
	for (i = 0; i < p->p_nrhs; i++)
		fprintf(stderr, " %s", g->g_name[p->p_rhs[i]]);
	fprintf(stderr, " %+d", p->p_prec);
}
#endif

void
print_item(g, it)
	grammar_t *g;
	item_t *it;
{
	int i;

	fprintf(stderr, "%s ->", g->g_name[it->i_prod->p_lhs]);
	for (i = 0; i < it->i_prod->p_nrhs; i++)
		fprintf(stderr, "%s%s", 
		    (i == it->i_dot) ? " . " : " ",
		    g->g_name[it->i_prod->p_rhs[i]]);
	fprintf(stderr, "%s; %s", 
		((it->i_prod->p_nrhs == it->i_dot) ? " . " : " "),
		g->g_name[it->i_follow]);
}

#ifndef NDEBUG
void
print_state(g, st)
	grammar_t *g;
	state_t *st;
{
	int i;
	item_t *it;
	int comma;
	symbol_t s;

	fprintf(stderr, "%% STATE %d\n", st->s_id);
	for (i=0,it=st->s_item; it; i++,it=it->i_next) {
	    fprintf(stderr, "%c", (i < st->s_nkitem) ? '|' : ' ');
	    print_item(g, it);
	    if (IS_REDUCE(it)) {
		for (comma = s = 0; s < g->g_nonterminals; s++) {
		    if (st->s_reduce[s] == it->i_prod) {
			if (comma == 0) {
				fprintf(stderr, "\t\treduce [");
				comma = 1;
			} else
				fprintf(stderr, ", ");
			fprintf(stderr, "%s", g->g_name[s]);
		    }
		}
		if (comma)
			fprintf(stderr, "]");
	    } else {
		s = it->i_prod->p_rhs[it->i_dot];
		if (st->s_shift[s])
		    fprintf(stderr, "\t\tshift [%s] -> %d", g->g_name[s],
		        st->s_shift[s]->s_id);
	    }
	    fprintf(stderr, "\n");
	}
}
#endif
