/*	$Id: symbol.c,v 1.1.1.1 1999/10/05 23:58:37 leonard Exp $	*/
/* Copyright 1999, David Leonard. The University of Queensland, Australia. */

#include <assert.h>
#include "symbol.h"

/* Empty a set. */
void
symsetempty(a)
	symset_t *a;
{
	memset(a->bits, '\0', sizeof a->bits);
}

/* Compare two sets for equality. */
int
symsetcmp(a, b)
	const symset_t *a, *b;
{
	return memcmp(a->bits, b->bits, sizeof a->bits);
}

/* Compare set for emptiness. */
int
symsetisempty(a)
	const symset_t *a;
{
	static const symset_t empty;

	return symsetcmp(&empty, a);
}

/* Insert a symbol into a set. */
void
symsetadd(a, sym)
	symset_t *a;
	const symbol_t sym;
{
	a->bits[sym/NBITS] |= (1 << (sym % NBITS));
}

/* Remove a symbol from a set. */
void
symsetdel(a, sym)
	symset_t *a;
	const symbol_t sym;
{
	a->bits[sym/NBITS] &= ~(1 << (sym % NBITS));
}

/* Test if a symbol is in a set. */
int
symsetin(a, sym)
	const symset_t *a;
	const symbol_t sym;
{
	return a->bits[sym/NBITS] & (1 << (sym % NBITS));
}

/* Add all symbols from b into set a. */
void
symsetunion(a, b)
	symset_t *a;
	const symset_t *b;
{
	int i;

	for (i = 0; i < sizeof a->bits; i++)
		a->bits[i] |= b->bits[i];
}

/* Efficiently find the next symbol in a set. */
symbol_t
symsetnext(a, start)
	const symset_t *a;
	symbol_t start;
{
	_BITTYPE c;
	_BITTYPE b;
	int i, pos;

	if (start == NO_SYMBOL) {
		pos = 0;
		i = 0;
		b = 1;
		c = a->bits[0];
	} else {
		pos = start + 1;
		i = pos / NBITS;
		b = 1 << pos % NBITS;
		c = a->bits[i] & ~(b - 1);
	}
	/* Quickly skip over spaces */
	while (c == 0) {
		/* If this is the last chunk, then there is nothing left. */
		if (i >= (MAXSYMS / NBITS) - 1)
			return NO_SYMBOL;
		i++;
		b = 1;
		pos = i * NBITS;
		c = a->bits[i];
	}
	/* slowly work way up towards the bit */
	assert(c != 0);
	while ((c & b) == 0) {
		b <<= 1;
		pos++;
	}
	return pos;
}

/* Find intersection of a and b; return true if non-empty */
int
symsetisect(a, b, res)
	const symset_t *a, *b;
	symset_t *res;
{
	int i;
	int ret = 0;

	for (i = 0; i < MAXSYMS / NBITS; i++) 
		if ((res->bits[i] = a->bits[i] & b->bits[i]))
			ret = 1;
	return ret;
}
